#include <stdlib.h>
#include <stdint.h>
#include <assert.h>

unsigned int nondet_uint();

#define assert2(x, y) __CPROVER_assert(x, y)
#define assume(x) __CPROVER_assume(x)

#ifndef V
#define V 2
#endif

#ifndef C
#define C 2
#endif

unsigned int voting(unsigned int votes[V][C+1]) {
    // V x C matrix of the according candidates for each preference of the according voter (votes[i][j] = 0 if not ranked)
    // (i.e. voter v ranks candidate c on preference rank p translates to "votes[v-1][p] == c")
    //assert (0 < C);
    //assert (0 < V);
    unsigned int res[C + 1];
    unsigned int elim[C];
    unsigned int i = 0;
    unsigned int j = 0;
    unsigned int j_prime = 0;
    unsigned int k = 0;
    unsigned int l = 0;
    unsigned int n = 0;
    unsigned int quota = V / 2;

    for (i = 0; i < V; i++) {
	assume (votes[i][1] != 0);
	for (j = 1; j <= C; j++) {
	    assume (0 <= votes[i][j]);
	    assume (votes[i][j] <= C);
	    for (j_prime = 1; j_prime <= C; j_prime++) {
		if ((votes[i][j] != 0) && (j != j_prime)) {
		    // for any voter and any two differing preference ranks with ranked candidates, the ranked candidates differ
		    assume (votes[i][j] != votes[i][j_prime]);
		}
		if ((votes[i][j] == 0) && (j <= j_prime)) {
		    // if one for one preference rank there is no candidate, the same applies for all consecutive ranks
		    assume (votes[i][j_prime] == 0);
		}
	    }
	}
    }

    for (i = 0; i <= C; i++) {
	res[i] = 0;
    }
    for (i = 0; i < V; i++) {
	res[votes[i][1]]++;
    }
    unsigned int max = 0;
    unsigned int elect = 0;
    for (i = 1; i <= C; i++) {
	if (max < res[i]) {
	    max = res[i];
	    elect = i;
	}
    }

    if (elect != 0 && quota < res[elect]) {
	return elect;
    } else {
	// second round
	// determine new maximum of votes for runoff election (second round)
	for (i = 0; i <= C; i++) {
	    elim[i] = 0;
	}
	res[elect] = 0;
	max = 0;
	for (i = 1; i <= C; i++) {
	    if (max < res[i]) {
		max = res[i];
	    }
	}

	n = 0;
	// all candidates not participating in the second round are stored in the array elim, there are n of them
	for (i = 1; i <= C; i++) {
	    if (res[i] < max && i != elect) {
		elim[n] = i;
		n++;
	    }
	}

	// eliminate other candidates
	for (i = 0; i < n; i++) {
	    for (j = 0; j < V; j++) {
		for (k = 1; k <= C; k++) {
		    if (votes[j][k] == elim[i]) {
			for (l = k; l < C; l++) {
			    votes[j][l] = votes[j][l + 1];
			}
			votes[j][C] = 0;
		    }
		}
	    }
	}
	// vote counting for second round
	for (i = 0; i <= C; i++) {
	    res[i] = 0;
	}
	for (i = 0; i < V; i++) {
	    res[votes[i][1]]++;
	}
	max = 0;
	elect = 0;
	// determine winner of second round
	for (i = 1; i <= C; i++) {
	    if (max < res[i]) {
		max = res[i];
		elect = i;
	    } else {
		if (max == res[i]) {
		    elect = 0;
		}
	    }
	}
	return elect;
    }
}
